<?php

	/*
	Plugin Name: Shipping Discount Based On Cart Subtotal
	Plugin URI: https://www.pluginhive.com/product/woocommerce-table-rate-shipping-pro-plugin/
	Description: This plugin will provide Percentage Discount on Shipping Rates for each Shipping Method, based on Cart Subtotal value.
	Version: 1.0
	Author: PluginHive
	Author URI: https://pluginhive.com/
	Text Domain: ph_shipping_pro_discount_on_shipping_rate

	WC requires at least: 2.6.0
	WC tested up to: 3.5.7

	Supported Plugin(s): WooCommerce Table Rate Shipping Pro Plugin

	*/

	if ( ! defined( 'ABSPATH' ) ) {
		exit;	// Exit if accessed directly
	}

	function ph_shipping_discount_on_cart_subtotal_activation_check(){

		if ( !is_plugin_active('woocommerce-shipping-pro/woocommerce-shipping-pro.php') ){
			deactivate_plugins( basename( __FILE__ ) );
			wp_die( __("Oops! You tried installing the Shipping Discount Based On Cart Subtotal without activating WooCommerce Shipping Pro with Table Rate. Kindly activate <b>WooCommerce Shipping Pro with Table Rate</b> and then try again", "ph_shipping_pro_discount_on_shipping_rate" ), "", array('back_link' => 1 ));
		}
	}

	register_activation_hook( __FILE__, 'ph_shipping_discount_on_cart_subtotal_activation_check' );

	if ( in_array( 'woocommerce-shipping-pro/woocommerce-shipping-pro.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ){

		function ph_shipping_pro_shipping_discount_on_cart_subtotal() {

			if ( ! class_exists( 'ph_shipping_discount_on_cart_subtotal' ) ){
				class ph_shipping_discount_on_cart_subtotal extends WC_Shipping_Method{

					public function __construct(){
						$this->id                 = 'ph_shipping_pro_discount_on_shipping_rate';
						$this->method_title       = __( 'Shipping Rate Discount', 'ph_shipping_pro_discount_on_shipping_rate');
						$this->method_description = __( 'Provide Percentage Discount on Shipping Rates for each Shipping Method, based on Cart Subtotal value.', 'ph_shipping_pro_discount_on_shipping_rate');

						$pro_settings = get_option('woocommerce_wf_woocommerce_shipping_pro_settings');
						$this->pro_settings = !empty($pro_settings)? $pro_settings : array();
						
						add_action( 'admin_enqueue_scripts', array($this,'ph_admin_enqueue'));

						$this->init();

						add_filter( 'woocommerce_package_rates', array( $this, 'ph_add_discount_on_shipping_rate'), 15, 2 );
					}

					function init() {

						$this->init_form_fields();
						$this->init_settings();

						add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
					}

					function init_form_fields() {

						$this->form_fields = array(

							'discount_rates' => array(
								'type'			=>	'discount_rate',
							)					

						);
					}

					public function generate_discount_rate_html()
					{
						$this->shipping_rules = array();
						if( isset($this->pro_settings['rate_matrix']) ){
							foreach ($this->pro_settings['rate_matrix'] as $key => $value) {
								$this->shipping_rules[$key] = $value['shipping_name'];
							}
						}
						
						$this->shipping_rules = array_unique($this->shipping_rules);

						$tool_tip_icon = plugins_url("woocommerce/assets/images/help.png");

						ob_start();
						?>
						<style type="text/css">
							.widefat th input, .widefat td input
							{
								margin: 0px;
								border-radius: 5px;
							}

							.widefat .check-column
							{
								padding: 10px !important;
								vertical-align: middle;
								text-align: right;
							}

							.discount_rate_table th, .discount_rate_table td
							{
								text-align: center;
								padding: 15px;
							}

							.ph-tooltip
							{
								position: relative;
								top: 4px;
							}

							.ph-tooltip .ph-tooltiptext
							{
								visibility: hidden;
								width: 150px;
								background-color: #333;
								color: #fff;
								text-align: center;
								border-radius: 3px; 
								padding: .618em 1em;
								font-size: .8em;
								position: absolute;
								z-index: 1;
							}
							
							.ph-tooltip:hover .ph-tooltiptext
							{
								visibility: visible;
							}

						</style>

						<table class="widefat posts discount_rate_table">

							<thead>
								<tr>
									<th class="check-column"><input type="checkbox" /></th>
									<th>
										Shipping Rules
										<?php
										echo '<span class="ph-tooltip"><img src="'.$tool_tip_icon.'" height="18" width="18" /><span class="ph-tooltiptext">Choose Shipping Rule For Discount</span></span>'
										?>	
									</th>
									<th colspan="2">
										Price Range
										<?php
										echo '<span class="ph-tooltip"><img src="'.$tool_tip_icon.'" height="18" width="18" /><span class="ph-tooltiptext">Cart Subtotal Range for which you want to provide Discount</span></span>'
										?>
									</th>
									<th>Discount (%)
										<?php
										echo '<span class="ph-tooltip"><img src="'.$tool_tip_icon.'" height="18" width="18" /><span class="ph-tooltiptext">Discount on Shipping Rate</span></span>'
										?>
									</th>
								</tr>
							</thead>

							<tbody>
								<?php
								$this->discount_rates = ( isset($this->settings['discount_rates']) && !empty($this->settings['discount_rates']) ) ? $this->settings['discount_rates'] : array();

								if ( $this->discount_rates ) {
									foreach ( $this->discount_rates as $key => $discount_rate ) {

										?>
										<tr>
											<td class="check-column"><input type="checkbox" /></td>

											<td>
												<select name="shipping_method_name[<?php echo $key; ?>][]" multiple="multiple" class="wc-enhanced-select select2" id="multi">
													<option disabled value=""> Select Shipping Rules </option>';
													<?php
													foreach( $this->shipping_rules as $value ) {
														if( is_array($discount_rate['shipping_method_name']) && in_array( $value, $discount_rate['shipping_method_name']) )
														{
															echo "<option value='".$value."' selected>".$value."</option>";
														}
														else
														{
															echo "<option value='".$value."'>".$value."</option>";
														}
													}
													?>
												</select>
											</td>

											<td><input type="number" name="min_cart_subtotal[<?php echo $key; ?>]" value="<?php echo esc_attr( $discount_rate['min_cart_subtotal'] ); ?>" placeholder="From" min="0" step="any"></td>
											<td><input type="number" name="max_cart_subtotal[<?php echo $key; ?>]" value="<?php echo esc_attr( $discount_rate['max_cart_subtotal'] ); ?>" placeholder="To" min="0" step="any"></td>
											<td><input type="number" name="shipping_discount[<?php echo $key; ?>]" value="<?php echo esc_attr( $discount_rate['shipping_discount'] ); ?>" placeholder="Discount" min="0" step="any"></td>

										</tr>
									<?php }
								}
								?>

							</tbody>

							<tfoot>
								<tr>
									<th colspan="5" style="text-align: right;">
										<a href="#" class="button insert"><?php _e( 'Add New'); ?></a>
										<a href="#" class="button remove"><?php _e( 'Remove'); ?></a>
									</th>
								</tr>
							</tfoot>
						</table>

						<?php
						return ob_get_clean();
					}

					public function validate_discount_rate_field( $key =array() ) {

						$discount_rates = array();

						$shipping_method_name	= isset( $_POST['shipping_method_name'] ) ? $_POST['shipping_method_name'] : array();
						$min_cart_subtotal	= isset( $_POST['min_cart_subtotal'] ) ? $_POST['min_cart_subtotal'] : array();
						$max_cart_subtotal	= isset( $_POST['max_cart_subtotal'] ) ? $_POST['max_cart_subtotal'] : array();
						$shipping_discount	= isset( $_POST['shipping_discount'] ) ? $_POST['shipping_discount'] : array();

						if( !empty( array_keys( $shipping_method_name )) )
						{
							for ( $i = 0; $i <= max( array_keys( $shipping_method_name ) ); $i ++ ) {

								if (empty( $shipping_method_name[ $i] ) )
									continue;
								
								$discount_rates[] = array(
									'shipping_method_name'	=> $shipping_method_name[$i],
									'min_cart_subtotal'	=> floatval( $min_cart_subtotal[ $i ] ),
									'max_cart_subtotal'	=> floatval( $max_cart_subtotal[ $i ] ),
									'shipping_discount'	=> floatval( $shipping_discount[ $i ] ),
								);
							}
						}
						return $discount_rates;
					}

					function ph_admin_enqueue()
					{
						$this->shipping_rules = array();
						if( $this->pro_settings['rate_matrix'] )
						{
							foreach ($this->pro_settings['rate_matrix'] as $key => $value) {
								$this->shipping_rules[$key] = $value['shipping_name'];
							}
						}

						$this->shipping_rules = array_unique($this->shipping_rules);
						
						wp_enqueue_script('admin_shipping_rules', plugins_url('/resources/admin-script.js', __FILE__ ));
						wp_localize_script( "admin_shipping_rules", "shipping_rules", $this->shipping_rules );

					}

					
					public	function ph_add_discount_on_shipping_rate( $available_shipping_methods, $package )
					{
						$cart_subtotal  = 	!empty(WC()->cart->subtotal) ? WC()->cart->subtotal : 0;
						$discount_rates = ( isset($this->settings['discount_rates']) && !empty($this->settings['discount_rates']) ) ? $this->settings['discount_rates'] : array();

						$package_hash=md5(json_encode($package)).md5(json_encode($discount_rates));

						if( !empty(get_transient($package_hash)) && get_transient($package_hash) != null)
						{	
							return get_transient( $package_hash );
						}


						if( !empty($cart_subtotal) && is_array($discount_rates) )
						{	
							foreach ($discount_rates as $key => $discount) {

								if( $cart_subtotal > $discount['min_cart_subtotal'] && $cart_subtotal <= $discount['max_cart_subtotal'] ){

									foreach ($available_shipping_methods as $shipping_method => $value) {

										$shipping_rule = $value->get_label();
										$shipping_rule_id = $value->get_method_id();

										if( in_array( $shipping_rule, $discount['shipping_method_name']) && $shipping_rule_id == 'wf_woocommerce_shipping_pro' )
										{	
											$percent = round(($discount['shipping_discount']/100),2);
											$adjustment_rate 	= $value->get_cost() * floatval( $percent );
											$value->cost 		-= $adjustment_rate;
										}

									}
								}
							}
						}	
						set_transient($package_hash,$available_shipping_methods,2*24*60*60); 

						return $available_shipping_methods;
					}	

				}
			}
		}

		add_action( 'woocommerce_shipping_init', 'ph_shipping_pro_shipping_discount_on_cart_subtotal' );

		function add_ph_shipping_pro_table_rate_addon( $methods ) {
			$methods['ph_shipping_pro_shipping_discount_on_cart_subtotal'] = 'ph_shipping_discount_on_cart_subtotal';
			return $methods;
		}

		add_filter( 'woocommerce_shipping_methods', 'add_ph_shipping_pro_table_rate_addon' );

		// Settings
		function settings_page( $links ) 
		{
			$settings_link = '<a href="admin.php?page=wc-settings&tab=shipping&section=ph_shipping_pro_discount_on_shipping_rate">Settings</a>';
			array_push( $links, $settings_link );
			return $links;
		}

		add_filter( "plugin_action_links_".plugin_basename(__FILE__), 'settings_page');

	}
	
